# -*- coding: utf-8 -*-
"""
Created on Thu Mar  3 11:22:03 2022

@author: Wahb
"""
# In[0] - Importation de bibliothèques

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from sklearn.neighbors import KNeighborsClassifier
from sklearn.preprocessing import MinMaxScaler
from sklearn.model_selection import train_test_split 
from sklearn.metrics import confusion_matrix

# In[1] - Prétraitement de données

## Importation de jeu de données
data = pd.read_excel(r'C:/Users/2016-1951/Desktop/5vs7.xlsx')   # importer le fichier de données excel
noms_params = data.columns

## Normalisation de jeu de données
data = data.to_numpy()                                                                              # convertir en NumPy array 
scaling = MinMaxScaler()                                                                            # mettre toutes les valeurs dans l'intervalle [0,1]. 
data = scaling.fit_transform(data)

## Fractionnement des données 70%-20%-10%
X = data[:,:-1]                                                                                     # définir les paramètres/attributs
y = data[:,-1]                                                                                      # définir les étiquettes (dernière colonne)
X_train, X_test, y_train, y_test = train_test_split(X, y,                                           # définir le jeu de test
                            test_size = 2/10, random_state = 42)                                    
                              

# In[2] - Classification par KNN

## Entraînement et réglage du modèle 
Kmax = int(np.sqrt(len(X)))+1                                                                       # la valeur maximale de K à tester
Acc_train = []; Acc_test = []
print("\nPrécision de prédiction du jeu de validation : ")
for K in range(3,Kmax,2):
    KNN = KNeighborsClassifier(n_neighbors=K)                                                       # un seul hyperparamètre à optimiser : le nombre K des voisins
    KNN.fit(X_train, y_train)
    Acc_train.append(KNN.score(X_train, y_train)) 
    Acc_test.append(KNN.score(X_test, y_test))                                                       # précision de prédiction du jeu de validation
    print("  Pour K =",K," : {:.2%}".format(Acc_test[-1]))

# In[3] - Figure : évolution de la précision de prédiction en fonction du paramètre K

plt.figure(figsize=(8,6))
plt.plot(range(3, Kmax,2), Acc_train, marker='o', c='red')
plt.plot(range(3, Kmax,2), Acc_test, marker='o', c='blue')
plt.xlabel('Nombre K de voisins', size = 12)
plt.ylabel('Précision de prédiction', size = 12)
plt.title("Précison de prédiction en fonction du paramètre K", size = 16)
plt.legend(["Précison apprentissage","Précision validation"])
plt.grid(linestyle=':')
plt.show()

# In[4] - sélectionner le K optimal

## pour choisir le K optimal qui maximise la précision de prédiction du jeu de validation
indices_dev = [i for i in range(len(Acc_test)) if Acc_test[i]==max(Acc_test)]
K_optimal = 2*indices_dev[0]+3

# In[5] - Modèle KNN final

KNN = KNeighborsClassifier(n_neighbors=K_optimal)                                                       # un seul hyperparamètre à définir : le nombre K des voisins
KNN.fit(X_train, y_train)                                                                               # précision de prédiction du jeu de test

print("\nK optimal =",K_optimal," : précision de prédiction du jeu d'apprentissage : {:.2%}".format(KNN.score(X_train, y_train)))
print("K optimal =",K_optimal," : précision de prédiction du jeu de test : {:.2%}\n".format(KNN.score(X_test, y_test)))

# In[6] -  Matrice de confusion

y_pred = KNN.predict(X_test)
Mat_conf = pd.DataFrame(data=confusion_matrix(y_test, y_pred),
             columns=["Positif réel", "Négatif réel"], index= ["Positif prédit", "Négatif prédit"])
print("Matrice de confusion :\n", Mat_conf)
